@@ -50,11 +50,13 @@
 #define MIN_TTY_CONSOL_NO			1
 #define MAX_TTY_CONSOL_NO			10
 
+#define MIN_BRAILLE_PORT 			1
+#define MAX_BRAILLE_PORT 			10
 
 #define DEFAULT_BRAILLE_ATTRIBUTES		8
 #define DEFAULT_BRAILLE_STYLE 			"8"
 #define DEFAULT_BRAILLE_CURSOR_STYLE		"underline"
-#define DEFAULT_BRAILLE_TRANSLATION 		"GE"
+#define DEFAULT_BRAILLE_TRANSLATION 		"de"
 #define DEFAULT_BRAILLE_DEVICE 			"VARIO20"
 #define DEFAULT_BRAILLE_DEVICE_INDEX 		0
 #define DEFAULT_BRAILLE_DEVICE_COUNT		0
@@ -64,6 +66,18 @@
 #define DEFAULT_BRAILLE_OPTICAL_SENSOR 		0
 #define DEFAULT_BRAILLE_POSITION_SENSOR		2
 /*____________________________< /BRAILLE_DEFAULTS>___________________________*/
+/*____________________________< BRAILLE_MONITOR_DEFAULTS>___________________________*/
+#define DEFAULT_BRAILLE_MONITOR_PANEL_POSITION   	"TOP"
+#define DEFAULT_BRAILLE_MONITOR_MODE			"normal"
+#define DEFAULT_BRAILLE_MONITOR_DOT78_COLOR		"#000000007D00"
+#define DEFAULT_BRAILLE_MONITOR_DOT7_COLOR		"#7D0000000000"
+#define DEFAULT_BRAILLE_MONITOR_DOT8_COLOR		"#00007D000000"
+#define DEFAULT_BRAILLE_MONITOR_LINE			2
+#define DEFAULT_BRAILLE_MONITOR_COLUMN			40
+#define DEFAULT_BRAILLE_MONITOR_USE_THEME		FALSE
+#define DEFAULT_BRAILLE_MONITOR_IS_PANEL		TRUE
+#define DEFAULT_BRAILLE_MONITOR_FONT_SIZE		18
+/*____________________________< /BRAILLE_MONITOR_DEFAULTS>___________________________*/
 /*____________________________< SRCORE_DEFAULTS>_____________________________*/
 #define DEFAULT_SRCORE_MAGNIF_ACTIVE		FALSE
 #define DEFAULT_SRCORE_SPEECH_ACTIVE		TRUE
@@ -95,19 +109,19 @@
 #define MIN_SPEECH_VOLUME 			0
 
 #define DEFAULT_SPEECH_PREEMPT 			TRUE
-#define DEFAULT_SPEECH_RATE 			80
-#define DEFAULT_SPEECH_PITCH 			80
+#define DEFAULT_SPEECH_RATE 			25
+#define DEFAULT_SPEECH_PITCH 			12
 #define DEFAULT_SPEECH_VOLUME 			100
 #define DEFAULT_SPEECH_COUNT_TYPE		"NONE"
 #define DEFAULT_SPEECH_VOICE_COUNT		0
 #define DEFAULT_SPEECH_CAPITAL			CAPITAL_IGNORE
-#define DEFAULT_SPEECH_PUNCTUATION		"SAME"
+#define DEFAULT_SPEECH_PUNCTUATION		"IGNORE"
 #define DEFAULT_SPEECH_TEXT_ECHO		"CHARACTER"
 #define DEFAULT_SPEECH_MODIFIERS		"NONE"
 #define DEFAULT_SPEECH_CURSORS			"NONE"
 #define DEFAULT_SPEECH_SPACES			"ALL"
 #define DEFAULT_SPEECH_ENGINE_VOICE		"V0 Adult Male 1 - IBM Viavoice GNOME Speech Driver"
-#define DEFAULT_SPEECH_ENGINE_DRIVER 		"IBM_Viavoice_GNOME_Speech_Driver"
+#define DEFAULT_SPEECH_ENGINE_DRIVER 		"IBM Viavoice GNOME Speech Driver"
 #define DEFAULT_SPEECH_DICTIONARY_ACTIVE	"NO"
 #define DEFAULT_SPEECH_DICTIONARY_ENTRY 	"USA<>United States Of America" 
 /*____________________________< /SPEECH_DEFAULTS>___________________________*/
@@ -119,6 +133,8 @@
 #define DEFAULT_PRESENTATION_DEVICE 		"braille"
 #define DEFAULT_PRESENTATION_ROLE		"generic"
 #define DEFAULT_PRESENTATION_EVENT		"generic"
+
+#define DEFAULT_PRESENTATION_ACTIVE_SETTING_NAME 	"default"
 /*____________________________< /PRESENTATION_DEFAULTS>___________________________*/
 /*____________________________< MAGNIFIER_DEFAULTS>___________________________*/
 #define MAX_ZOOM_FACTOR_X  			15
@@ -143,7 +159,6 @@
 #define DEFAULT_MAGNIFIER_CURSOR		TRUE
 #define DEFAULT_MAGNIFIER_CURSOR_NAME		"default"
 #define DEFAULT_MAGNIFIER_CURSOR_SIZE		32
-#define DEFAULT_MAGNIFIER_CURSOR_MAG		FALSE
 #define DEFAULT_MAGNIFIER_CURSOR_SCALE		FALSE
 #define DEFAULT_MAGNIFIER_CURSOR_COLOR		0
 
@@ -160,8 +175,8 @@
 #define DEFAULT_MAGNIFIER_BORDER_COLOR		0
 
 #define DEFAULT_MAGNIFIER_ID			"generic_zoomer"
-#define DEFAULT_MAGNIFIER_SOURCE		""
-#define DEFAULT_MAGNIFIER_TARGET		""
+#define DEFAULT_MAGNIFIER_SOURCE		":0.0"
+#define DEFAULT_MAGNIFIER_TARGET		":0.0"
 
 #define DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY	2
 #define DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK	TRUE
@@ -172,12 +187,16 @@
 #define DEFAULT_MAGNIFIER_TRACKING		"focus"
 #define DEFAULT_MAGNIFIER_MOUSE_TRACKING	"mouse-push"
 
+#define DEFAULT_MAGNIFIER_ALIGNMENT_X		"auto"
+#define DEFAULT_MAGNIFIER_ALIGNMENT_Y		"auto"
+
+
 #define DEFAULT_MAGNIFIER_VISIBLE		TRUE
 
 #define DEFAULT_MAGNIFIER_PANNING		FALSE	
 
-#define DEFAULT_SCREEN_MIN_SIZE			1
-#define DEFAULT_SCREEN_MAX_SIZE			1024
+#define DEFAULT_SCREEN_MIN_SIZE			0
+#define DEFAULT_SCREEN_MAX_SIZE			1023
 
 /*____________________________</MAGNIFIER_DEFAULTS>___________________________*/
 /*___________________________________________________________________________*/
@@ -203,6 +222,8 @@
 #define SRCORE_EXIT_KEY				"exit"
 #define SRCORE_EXIT_ACK_KEY			"exit_ack"
 
+#define SRCORE_INITIALIZATION_END		"init_end"
+
 #define SRCORE_UI_COMMAND			"ui_command"
 
 #define SRCORE_FIND_TYPE			"find_type"
@@ -236,7 +257,6 @@
 #define MAGNIFIER_CURSOR			"cursor"
 #define MAGNIFIER_CURSOR_NAME 			"cursor_name"
 #define MAGNIFIER_CURSOR_SIZE 			"cursor_size"
-#define MAGNIFIER_CURSOR_MAG    		"cursor_mag"
 #define MAGNIFIER_CURSOR_SCALE    		"cursor_scale"
 #define MAGNIFIER_CURSOR_COLOR			"cursor_color"
 
@@ -266,6 +286,9 @@
 #define MAGNIFIER_TRACKING			"tracking"
 #define MAGNIFIER_MOUSE_TRACKING		"mouse_tracking"
 
+#define MAGNIFIER_ALIGNMENT_X			"alignment_x"
+#define MAGNIFIER_ALIGNMENT_Y			"alignment_y"
+
 #define MAGNIFIER_VISIBLE 			"visible"
 #define MAGNIFIER_PANNING	  		"panning"
 /*____________________________</MAGNIFIER>____________________________________*/
@@ -307,11 +330,28 @@
 /*____________________________</COMMAND MAP>___________________________________*/
 /*____________________________<PRESENTATION>___________________________________*/
 #define PRESENTATION_CHANGES_END 		"/changes_end"
+#define PRESENTATION_SETTING_LIST		"presentation_list"
+#define PRESENTATION_ACTIVE_SETTING		"active_setting"
 /*____________________________</PRESENTATION>__________________________________*/
+/*____________________________</BRAILLE MONITOR>__________________________________*/
+#define BRAILLE_MONITOR_PANEL_POSITION_KEY 	"brlmon_position"
+#define BRAILLE_MONITOR_COLUMN_KEY	 	"cell_column"
+#define BRAILLE_MONITOR_LINE_KEY		"cell_line"
+#define BRAILLE_MONITOR_MODE_KEY		"display_mode"
+#define BRAILLE_MONITOR_DOT7_KEY 	 	"dot7"
+#define BRAILLE_MONITOR_DOT8_KEY 	 	"dot8"
+#define BRAILLE_MONITOR_DOT78_KEY 	 	"dot78"
+#define BRAILLE_MONITOR_USE_THEME_KEY		"use_theme_color"
+#define BRAILLE_MONITOR_IS_PANEL_KEY		"is_brlmon_panel"
+#define BRAILLE_MONITOR_FONT_SIZE_KEY		"font_size"
+/*____________________________</BRAILLE MONITOR>__________________________________*/
 /*____________________________<SECTIONS>__________________________________*/
 
 #define CONFIG_PATH 				"/apps/gnopernicus/"
 
+#define BRAILLE_MONITOR_PATH 			"brlmon"
+#define BRAILLE_MONITOR_KEY_PATH 		"brlmon/"
+
 #define BRAILLE_PATH 				"braille"
 #define BRAILLE_KEY_PATH			"braille/"
 
@@ -362,12 +402,12 @@ typedef enum {
  * types used for configuration
 **/
 typedef enum {
-    CFGT_BOOL	= 0,
-    CFGT_INT	= 1<<0,
-    CFGT_FLOAT	= 1<<1,
-    CFGT_STRING = 1<<2,
-    CFGT_LIST   = 1<<3,
-    CFGT_UNSET	= 1<<4
+    CFGT_BOOL	= GCONF_VALUE_BOOL,
+    CFGT_INT	= GCONF_VALUE_INT,
+    CFGT_FLOAT	= GCONF_VALUE_FLOAT,
+    CFGT_STRING = GCONF_VALUE_STRING,
+    CFGT_LIST   = GCONF_VALUE_LIST,
+    CFGT_UNSET	= GCONF_VALUE_INVALID
 } SRConfigTypesEnum;
 
 typedef struct {
@@ -411,7 +451,7 @@ srconf_free_slist (GSList *list);
  *	gconf_root_dir_path - the root dir of GConf
 **/
 gboolean 
-srconf_init(SRConfCB srconfcb, const gchar *gconf_root_dir_path);
+srconf_init(SRConfCB srconfcb, const gchar *_gconf_root_dir_path, const gchar *config_source);
 
 /**
  * srconf_terminate:
@@ -532,4 +572,6 @@ srconf_set_data(const gchar *key, SRConfigTypesEnum conftype, const gpointer dat
 gboolean
 srconf_unset_key (const gchar *key,const gchar *section);
 
+gchar*
+srconf_presentationi_get_chunk (const gchar *device_role_event);
 #endif /*_LIBSRCONF_H */
