@@ -116,8 +116,9 @@ src_magnifier_automatic_panning (gpointer data)
     gchar 		*reason_, *magout = NULL;
     gboolean		stop_panning = FALSE;
     int 		step_x  = 2;/*TBR*/
-
-    sro_add_reference (obj);
+    
+    if (obj)
+	sro_add_reference (obj);
 
     if (last != data)
     {
@@ -294,7 +295,7 @@ src_magnifier_automatic_panning (gpointer data)
 static gboolean
 src_magnifier_panning (SRObject *obj)
 {
-    static guint 	panning_handle = 0;
+    static guint 	panning_handle = 0;		
     
     if (panning_handle)
     {
@@ -307,8 +308,8 @@ src_magnifier_panning (SRObject *obj)
     old_panning = obj;
     if (old_panning)
 	sro_add_reference (old_panning);
-
-    if (panning)
+	
+    if (panning && old_panning)
     {
 	panning_handle = gtk_timeout_add (100,
 					src_magnifier_automatic_panning,
@@ -323,7 +324,7 @@ src_magnifier_start_panning (SRObject *obj)
 {
 /*    SRObjectRoles role;*/
 
-    sru_assert (obj);
+    sru_assert (obj);		
     
 /*    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
     if (role == SR_ROLE_TEXT)*/
@@ -334,7 +335,7 @@ src_magnifier_start_panning (SRObject *obj)
 	{
 	    if (strcmp (reason_, "focus:") == 0 ||
 		strcmp (reason_, "object:text-caret-moved") == 0)
-	    {
+	    {	
 		src_magnifier_panning (obj);
 	    }
 	    else
@@ -1056,6 +1057,8 @@ src_magnifier_config_get_defaults ()
     SRCMagnifier *magnifier = NULL;
 
     magnifier = src_magnifier_setting_new ();
+    sru_assert (magnifier);
+    	
     src_magnifier_load_defaults (magnifier);
     src_magnifier_config_set_defaults (magnifier);
     
@@ -1069,9 +1072,11 @@ src_magnifier_config_get_settings ()
     gchar  	 *xml 	    = NULL;
 
     magnifier = src_magnifier_setting_new ();
+    sru_assert (magnifier);
+    
     src_magnifier_load_values (magnifier);
 
-    xml       = src_magnifier_create_mml (magnifier);
+    xml = src_magnifier_create_mml (magnifier);
     src_magnifier_send (xml);
 
     src_magnifier_setting_free (magnifier);
@@ -1082,6 +1087,7 @@ src_magnifier_config_get_settings ()
 SRCMagnifier*
 src_magnifier_get_global_settings ()
 {
+    sru_assert (magnifier_initialized && src_magnifier);
     return src_magnifier;
 }
 
@@ -1092,6 +1098,8 @@ src_magnifier_set_cursor_state (gboolean cursor_state)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
     
     if (cursor_state != magnifier->cursor_state)
     {
@@ -1108,6 +1116,18 @@ src_magnifier_set_cursor_state (gboolean cursor_state)
     return FALSE;
 }
 
+gboolean
+src_magnifier_set_cursor_state_on_off ()
+{
+    SRCMagnifier *magnifier;
+    
+    magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+        
+    return src_magnifier_set_cursor_state (!magnifier->cursor_state);
+}
+
 gboolean
 src_magnifier_set_cursor_name (gchar *cursor_name)
 {
@@ -1115,8 +1135,10 @@ src_magnifier_set_cursor_name (gchar *cursor_name)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
-    
-    sru_assert (cursor_name && magnifier->cursor_name);
+    sru_assert (cursor_name && 
+		magnifier   && 
+		magnifier->cursor_name);
+    sru_assert (magnifier_initialized);
     
     if (strcmp(cursor_name, magnifier->cursor_name) != 0)
     {
@@ -1142,6 +1164,8 @@ src_magnifier_set_cursor_size (gint cursor_size)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
     
     cursor_size = (cursor_size > MAX_CURSOR_SIZE) ? MAX_CURSOR_SIZE : 
 		  (cursor_size < MIN_CURSOR_SIZE) ? MIN_CURSOR_SIZE : 
@@ -1162,6 +1186,21 @@ src_magnifier_set_cursor_size (gint cursor_size)
     return FALSE;
 }
 
+gboolean
+src_magnifier_adjust_cursor_size (gint offset)
+{
+    SRCMagnifier *magnifier;
+    gint cursor_size;
+    
+    magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
+    cursor_size = magnifier->cursor_size + offset;
+    
+    return src_magnifier_set_cursor_size (cursor_size);
+}
+
 gboolean
 src_magnifier_set_cursor_scale (gboolean cursor_scale)
 {
@@ -1169,6 +1208,8 @@ src_magnifier_set_cursor_scale (gboolean cursor_scale)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
     
     if (cursor_scale != magnifier->cursor_scale)
     {
@@ -1185,6 +1226,22 @@ src_magnifier_set_cursor_scale (gboolean cursor_scale)
     return FALSE;
 }
 
+gboolean
+src_magnifier_set_cursor_magnification_on_off ()
+{
+    SRCMagnifier *magnifier;
+    
+    magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
+    if (magnifier->cursor_state)
+    {
+        return src_magnifier_set_cursor_scale (!magnifier->cursor_scale);
+    }
+    return FALSE;
+}
+
 gboolean
 src_magnifier_set_cursor_color (gint32 cursor_color)
 {
@@ -1192,6 +1249,7 @@ src_magnifier_set_cursor_color (gint32 cursor_color)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier && magnifier_initialized);
     
     if (cursor_color != magnifier->cursor_color)
     {
@@ -1215,6 +1273,7 @@ src_magnifier_set_crosswire_state (gboolean crosswire_state)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier && magnifier_initialized);
     
     if (crosswire_state != magnifier->crosswire_state)
     {
@@ -1231,6 +1290,23 @@ src_magnifier_set_crosswire_state (gboolean crosswire_state)
     return FALSE;
 }
 
+gboolean
+src_magnifier_set_crosswire_on_off ()
+{	
+    SRCMagnifier *magnifier;
+    
+    magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+
+    if (magnifier->cursor_state)
+    {
+	return src_magnifier_set_crosswire_state (!magnifier->crosswire_state);
+    }
+    return FALSE;
+}
+
+
 gboolean
 src_magnifier_set_crosswire_clip (gboolean crosswire_clip)
 {
@@ -1238,6 +1314,7 @@ src_magnifier_set_crosswire_clip (gboolean crosswire_clip)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier && magnifier_initialized);
     
     if (crosswire_clip != magnifier->crosswire_clip)
     {
@@ -1254,6 +1331,24 @@ src_magnifier_set_crosswire_clip (gboolean crosswire_clip)
     return FALSE;
 }
 
+gboolean
+src_magnifier_set_crosswire_clip_on_off ()
+{
+    SRCMagnifier *magnifier;
+    
+    magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
+    if (magnifier->cursor_state &&
+	magnifier->crosswire_state)
+    {
+	return src_magnifier_set_crosswire_clip (!magnifier->crosswire_clip);
+    }
+    return FALSE;
+}
+
+
 gboolean
 src_magnifier_set_crosswire_size (gint crosswire_size)
 {
@@ -1261,6 +1356,8 @@ src_magnifier_set_crosswire_size (gint crosswire_size)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
     
     crosswire_size = (crosswire_size < MIN_CROSSWIRE_SIZE) ? MIN_CROSSWIRE_SIZE : 
 		     (crosswire_size > MAX_CROSSWIRE_SIZE) ? MAX_CROSSWIRE_SIZE :
@@ -1282,6 +1379,21 @@ src_magnifier_set_crosswire_size (gint crosswire_size)
     return FALSE;
 }
 
+gboolean
+src_magnifier_adjust_crosswire_size (gint offset)
+{
+    SRCMagnifier *magnifier;
+    gint crosswire_size;
+    
+    magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
+    crosswire_size = magnifier->crosswire_size + offset;
+    
+    return src_magnifier_set_crosswire_size (crosswire_size);
+}
+
 gboolean
 src_magnifier_set_crosswire_color (gint32 crosswire_color)
 {
@@ -1289,6 +1401,8 @@ src_magnifier_set_crosswire_color (gint32 crosswire_color)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
     
     if (crosswire_color != magnifier->crosswire_color)
     {
@@ -1312,7 +1426,9 @@ src_magnifier_set_zp_left (gint zp_left)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
-
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
     if (zp_left != magnifier->zp_left)
     {
 	magnifier->zp_left = zp_left;
@@ -1335,7 +1451,9 @@ src_magnifier_set_zp_top (gint zp_top)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
-
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
     if (zp_top != magnifier->zp_top)
     {
 	magnifier->zp_top = zp_top;
@@ -1358,7 +1476,9 @@ src_magnifier_set_zp_width (gint zp_width)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
-
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
     if (zp_width != magnifier->zp_right)
     {
 	magnifier->zp_right = zp_width;
@@ -1381,6 +1501,8 @@ src_magnifier_set_zp_height (gint zp_height)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
 
     if (zp_height != magnifier->zp_bottom)
     {
@@ -1404,8 +1526,12 @@ src_magnifier_set_target (gchar *target)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
 
-    sru_assert (target && magnifier->target);
+    sru_assert (target	  && 
+		magnifier && 
+		magnifier->target);
+    sru_assert (magnifier_initialized);
     
     if (strcmp(target, magnifier->target) != 0)
     {
@@ -1432,7 +1558,10 @@ src_magnifier_set_source (gchar *source)
     
     magnifier = src_magnifier_get_global_settings ();
 
-    sru_assert (source && magnifier->source);
+    sru_assert (source	  && 
+		magnifier && 
+		magnifier->source);
+    sru_assert (magnifier_initialized);
     
     if (strcmp(source, magnifier->source) != 0)
     {
@@ -1458,7 +1587,9 @@ src_magnifier_set_zoom_factor_x (gint zoom_factor_x)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
-
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
     if (zoom_factor_x != magnifier->zoom_factor_x)
     {
 	magnifier->zoom_factor_x = zoom_factor_x;
@@ -1473,6 +1604,58 @@ src_magnifier_set_zoom_factor_x (gint zoom_factor_x)
     return FALSE;
 }
 
+gboolean
+src_magnifier_increase_x_scale ()
+{
+    SRCMagnifier *magnifier;
+    gint zoom_factor;
+        
+    magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
+    zoom_factor = magnifier->zoom_factor_x;
+    
+    zoom_factor++;
+    
+    zoom_factor = zoom_factor > MAX_ZOOM_FACTOR_X ? MAX_ZOOM_FACTOR_X : zoom_factor;
+    
+    if (!magnifier->zoom_factor_lock)
+        src_magnifier_set_zoom_factor_x (zoom_factor);
+    else
+    {
+	src_magnifier_set_zoom_factor_y (zoom_factor);
+	src_magnifier_set_zoom_factor_x (zoom_factor);
+    }
+    return TRUE;
+}
+
+gboolean
+src_magnifier_decrease_x_scale ()
+{
+    SRCMagnifier *magnifier;
+    gint zoom_factor;
+        
+    magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
+    zoom_factor = magnifier->zoom_factor_x;
+    
+    zoom_factor--;
+    
+    zoom_factor = zoom_factor < MIN_ZOOM_FACTOR_X ? MIN_ZOOM_FACTOR_X : zoom_factor;
+    
+    if (!magnifier->zoom_factor_lock)
+        src_magnifier_set_zoom_factor_x (zoom_factor);
+    else
+    {
+    	src_magnifier_set_zoom_factor_y (zoom_factor);
+	src_magnifier_set_zoom_factor_x (zoom_factor);
+    }
+    return TRUE;
+}
+
 gboolean
 src_magnifier_set_zoom_factor_y (gint zoom_factor_y)
 {
@@ -1480,7 +1663,9 @@ src_magnifier_set_zoom_factor_y (gint zoom_factor_y)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
-
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
     if (zoom_factor_y != magnifier->zoom_factor_y)
     {
 	magnifier->zoom_factor_y = zoom_factor_y;
@@ -1496,13 +1681,67 @@ src_magnifier_set_zoom_factor_y (gint zoom_factor_y)
     return FALSE;
 }
 
+gboolean
+src_magnifier_increase_y_scale ()
+{
+    SRCMagnifier *magnifier;
+    gint zoom_factor;
+        
+    magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
+    zoom_factor = magnifier->zoom_factor_y;
+    
+    zoom_factor++;
+    
+    zoom_factor = zoom_factor > MAX_ZOOM_FACTOR_Y ? MAX_ZOOM_FACTOR_Y : zoom_factor;
+    
+    if (!magnifier->zoom_factor_lock)
+        src_magnifier_set_zoom_factor_y (zoom_factor);
+    else
+    {
+    	src_magnifier_set_zoom_factor_y (zoom_factor);
+	src_magnifier_set_zoom_factor_x (zoom_factor);
+    }	
+    return TRUE;
+}
+
+gboolean
+src_magnifier_decrease_y_scale ()
+{
+    SRCMagnifier *magnifier;
+    gint zoom_factor;
+        
+    magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
+    zoom_factor = magnifier->zoom_factor_y;
+    
+    zoom_factor--;
+    
+    zoom_factor = zoom_factor < MIN_ZOOM_FACTOR_Y ? MIN_ZOOM_FACTOR_Y : zoom_factor;
+    
+    if (!magnifier->zoom_factor_lock)
+        src_magnifier_set_zoom_factor_y (zoom_factor);
+    else
+    {
+    	src_magnifier_set_zoom_factor_y (zoom_factor);
+	src_magnifier_set_zoom_factor_x (zoom_factor);
+    }
+    return TRUE;
+}
+
 gboolean
 src_magnifier_set_zoom_factor_lock (gboolean zoom_factor_lock)
 {
     SRCMagnifier *magnifier;
     
     magnifier = src_magnifier_get_global_settings ();
-
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
     if (zoom_factor_lock != magnifier->zoom_factor_lock)
     {
 	magnifier->zoom_factor_lock = zoom_factor_lock;
@@ -1512,6 +1751,24 @@ src_magnifier_set_zoom_factor_lock (gboolean zoom_factor_lock)
     return FALSE;
 }
 
+gboolean
+src_magnifier_set_zoom_factor_lock_on_off ()
+{
+    SRCMagnifier *magnifier;
+    
+    magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
+    src_magnifier_set_zoom_factor_lock (!magnifier->zoom_factor_lock);
+    
+    if (magnifier->zoom_factor_lock)
+	src_magnifier_set_zoom_factor_x (magnifier->zoom_factor_x);
+	src_magnifier_set_zoom_factor_x (magnifier->zoom_factor_x);
+
+    return TRUE;
+}
+
 gboolean
 src_magnifier_set_invert_on_off (gboolean invert)
 {
@@ -1519,7 +1776,9 @@ src_magnifier_set_invert_on_off (gboolean invert)
     gchar *val = NULL;
     
     magnifier = src_magnifier_get_global_settings ();
-
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
     if (invert != magnifier->invert)
     {
 	magnifier->invert = invert;
@@ -1535,6 +1794,20 @@ src_magnifier_set_invert_on_off (gboolean invert)
     return FALSE;
 }
 
+gboolean 
+src_magnifier_set_invert_state ()
+{
+    SRCMagnifier *magnifier;
+        
+    magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
+    
+    src_magnifier_set_invert_on_off (!magnifier->invert);
+    
+    return TRUE;
+}
+
 gboolean
 src_magnifier_set_smoothing (gchar *smoothing)
 {
@@ -1543,8 +1816,11 @@ src_magnifier_set_smoothing (gchar *smoothing)
     
     magnifier = src_magnifier_get_global_settings ();
     
-    sru_assert (smoothing && magnifier->smoothing);
-
+    sru_assert (smoothing && 
+		magnifier && 
+		magnifier->smoothing);
+    sru_assert (magnifier_initialized);
+    
     if (strcmp (magnifier->smoothing, smoothing) != 0)
     {
 	g_free (magnifier->smoothing);
@@ -1570,7 +1846,10 @@ src_magnifier_set_mouse_tracking_mode (gchar *mouse_tracking)
     
     magnifier = src_magnifier_get_global_settings ();
     
-    sru_assert (mouse_tracking && magnifier->mouse_tracking);
+    sru_assert (mouse_tracking && 
+		magnifier      && 
+		magnifier->mouse_tracking);
+    sru_assert (magnifier_initialized);
     
     if (strcmp (magnifier->mouse_tracking, mouse_tracking) != 0)
     {
@@ -1595,6 +1874,8 @@ src_magnifier_set_panning_on_off (gboolean _panning)
     SRCMagnifier *magnifier;
     
     magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
 
     if (_panning != panning)
     {
@@ -1608,6 +1889,12 @@ src_magnifier_set_panning_on_off (gboolean _panning)
     return FALSE;
 }
 
+gboolean
+src_magnifier_set_panning_state()
+{
+    return src_magnifier_set_panning_on_off (!panning);
+}
+
 gboolean
 src_magnifier_set_alignment_x (gchar *alignment_x)
 {
@@ -1616,8 +1903,11 @@ src_magnifier_set_alignment_x (gchar *alignment_x)
     
     magnifier = src_magnifier_get_global_settings ();
 
-    sru_assert (alignment_x && magnifier->alignment_x);
-
+    sru_assert (alignment_x && 
+	        magnifier   &&
+		magnifier->alignment_x);
+    sru_assert (magnifier_initialized);
+    
     if (strcmp (magnifier->alignment_x, alignment_x) != 0)
     {
 	g_free (magnifier->alignment_x);
@@ -1642,7 +1932,10 @@ src_magnifier_set_alignment_y (gchar *alignment_y)
     
     magnifier = src_magnifier_get_global_settings ();
 
-    sru_assert (alignment_y && magnifier->alignment_y);
+    sru_assert (alignment_y && 
+		magnifier   && 
+		magnifier->alignment_y);
+    sru_assert (magnifier_initialized);
     
     if (strcmp (magnifier->alignment_y, alignment_y) != 0)
     {
@@ -1668,13 +1961,13 @@ src_magnifier_set_split_screen_horizontal (gboolean horiz_split)
     
     
     magnifier = src_magnifier_get_global_settings ();
+    sru_assert (magnifier);
+    sru_assert (magnifier_initialized);
     
     zp_left = zp_top = 0;
     zp_width  = gdk_screen_width()-1;
     zp_height = gdk_screen_height()/2;
-    
-    
-    
+        
     if (horiz_split)
     {
     
@@ -1733,4 +2026,4 @@ src_magnifier_set_split_screen_horizontal (gboolean horiz_split)
 	
     }
     return TRUE;
-}
\ No newline at end of file
+}
